@@ -4,9 +4,8 @@ from typing import NamedTuple
 from uuid import uuid4
 from pathlib import Path
 from itertools import zip_longest
-from programlib.agent import Agent
 
-from programlib import language_
+from programlib import Agent, language_, Terminal
 
 def correctness(expected_outputs, outputs):
     assert expected_outputs, 'expected_outputs is empty. Cannot test program correctness.'
@@ -26,14 +25,16 @@ class Program():
     Program object: represents a runnable program in some programming language
     """
 
-    def __init__(self, source, name=None, language='C++', 
+    def __init__(self, source=None, name=None, language='C++', 
                        workdir=Path(__file__).parent / 'programs'):
         self.language = language_(language)
+        self.term = Terminal()
 
         self.workdir = workdir
         self.name = name or str(uuid4())
         
-        self.language.write_source(self.workdir, self.name, source)
+        if source:
+            self.language.write_source(self.workdir, self.name, source)
         self.stdout, self.exitstatus = self.language.build(self.workdir, self.name)
 
         # Please send your opinions on 'not not' vs 'bool()' to dont@vadim.me
@@ -61,7 +62,7 @@ class Program():
 
         self.stdout, self.exitstatus = self.language.run(self.workdir, self.name, input_lines)
         assert force or not self.exitstatus, f'Exit status {self.exitstatus}'
-        return self.stdout.splitlines()
+        return repr(self.stdout).splitlines()
     
     def spawn(self, delimiter='\n'):
         """
